# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError as core_api_error_ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..requests.webhook_advanced_options import WebhookAdvancedOptionsParams
from ..types.api_error import ApiError as types_api_error_ApiError
from ..types.api_version_enum import ApiVersionEnum
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_dir import SortDir
from ..types.webhook_endpoint import WebhookEndpoint
from ..types.webhook_endpoint_create import WebhookEndpointCreate
from ..types.webhook_endpoint_event_type import WebhookEndpointEventType
from ..types.webhook_endpoint_status import WebhookEndpointStatus
from .types.webhook_endpoints_delete_response import WebhookEndpointsDeleteResponse
from .types.webhook_endpoints_list_response import WebhookEndpointsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWebhookEndpointsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        status: typing.Optional[WebhookEndpointStatus] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookEndpointsListResponse]:
        """
        List all webhook endpoints.

        Parameters
        ----------
        status : typing.Optional[WebhookEndpointStatus]
            Filter by endpoint status.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookEndpointsListResponse]
            List of webhook endpoints
        """
        _response = self._client_wrapper.httpx_client.request(
            "webhook_endpoints",
            method="GET",
            params={
                "status": status,
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointsListResponse,
                    construct_type(
                        type_=WebhookEndpointsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def create(
        self,
        *,
        url: str,
        name: str,
        enabled_events: typing.Sequence[WebhookEndpointEventType],
        api_version: ApiVersionEnum,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookEndpointCreate]:
        """
        Create a new webhook endpoint. The response includes a `signingSecret` that is only returned once — store it securely for verifying webhook signatures.

        The `enabledEvents` array specifies which global event types this endpoint should receive. Use the [Webhook Events](https://docs.extend.ai/2026-02-09/developers/api-reference/webhook-events) reference to see available event types.

        To subscribe to events scoped to a specific resource (e.g., a single extractor or workflow), use [Create Webhook Subscription](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/webhook/create-webhook-subscription) after creating the endpoint.

        Parameters
        ----------
        url : str
            The URL that webhook events will be sent to.

        name : str
            A human-readable name for the webhook endpoint.

        enabled_events : typing.Sequence[WebhookEndpointEventType]
            The list of global event types to subscribe to. Pass an empty array to create an endpoint with no global events (useful if you only plan to use resource-scoped subscriptions).

        api_version : ApiVersionEnum

        status : typing.Optional[WebhookEndpointStatus]

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookEndpointCreate]
            Successfully created webhook endpoint
        """
        _response = self._client_wrapper.httpx_client.request(
            "webhook_endpoints",
            method="POST",
            json={
                "url": url,
                "name": name,
                "status": status,
                "enabledEvents": enabled_events,
                "apiVersion": api_version,
                "advancedOptions": convert_and_respect_annotation_metadata(
                    object_=advanced_options, annotation=WebhookAdvancedOptionsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointCreate,
                    construct_type(
                        type_=WebhookEndpointCreate,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[WebhookEndpoint]:
        """
        Retrieve a webhook endpoint by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookEndpoint]
            Retrieved webhook endpoint
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpoint,
                    construct_type(
                        type_=WebhookEndpoint,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def update(
        self,
        id: str,
        *,
        url: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        enabled_events: typing.Optional[typing.Sequence[WebhookEndpointEventType]] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WebhookEndpoint]:
        """
        Update a webhook endpoint. Only the fields you include in the request body will be updated; omitted fields remain unchanged.

        The `apiVersion` of a webhook endpoint cannot be changed after creation.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to update.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        url : typing.Optional[str]
            The URL that webhook events will be sent to.

        name : typing.Optional[str]
            A human-readable name for the webhook endpoint.

        status : typing.Optional[WebhookEndpointStatus]

        enabled_events : typing.Optional[typing.Sequence[WebhookEndpointEventType]]
            The list of global event types to subscribe to.

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookEndpoint]
            Successfully updated webhook endpoint
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="POST",
            json={
                "url": url,
                "name": name,
                "status": status,
                "enabledEvents": enabled_events,
                "advancedOptions": convert_and_respect_annotation_metadata(
                    object_=advanced_options, annotation=WebhookAdvancedOptionsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpoint,
                    construct_type(
                        type_=WebhookEndpoint,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[WebhookEndpointsDeleteResponse]:
        """
        Delete a webhook endpoint and all of its subscriptions. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to delete.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WebhookEndpointsDeleteResponse]
            Successfully deleted webhook endpoint
        """
        _response = self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointsDeleteResponse,
                    construct_type(
                        type_=WebhookEndpointsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )


class AsyncRawWebhookEndpointsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        status: typing.Optional[WebhookEndpointStatus] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookEndpointsListResponse]:
        """
        List all webhook endpoints.

        Parameters
        ----------
        status : typing.Optional[WebhookEndpointStatus]
            Filter by endpoint status.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookEndpointsListResponse]
            List of webhook endpoints
        """
        _response = await self._client_wrapper.httpx_client.request(
            "webhook_endpoints",
            method="GET",
            params={
                "status": status,
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointsListResponse,
                    construct_type(
                        type_=WebhookEndpointsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def create(
        self,
        *,
        url: str,
        name: str,
        enabled_events: typing.Sequence[WebhookEndpointEventType],
        api_version: ApiVersionEnum,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookEndpointCreate]:
        """
        Create a new webhook endpoint. The response includes a `signingSecret` that is only returned once — store it securely for verifying webhook signatures.

        The `enabledEvents` array specifies which global event types this endpoint should receive. Use the [Webhook Events](https://docs.extend.ai/2026-02-09/developers/api-reference/webhook-events) reference to see available event types.

        To subscribe to events scoped to a specific resource (e.g., a single extractor or workflow), use [Create Webhook Subscription](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/webhook/create-webhook-subscription) after creating the endpoint.

        Parameters
        ----------
        url : str
            The URL that webhook events will be sent to.

        name : str
            A human-readable name for the webhook endpoint.

        enabled_events : typing.Sequence[WebhookEndpointEventType]
            The list of global event types to subscribe to. Pass an empty array to create an endpoint with no global events (useful if you only plan to use resource-scoped subscriptions).

        api_version : ApiVersionEnum

        status : typing.Optional[WebhookEndpointStatus]

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookEndpointCreate]
            Successfully created webhook endpoint
        """
        _response = await self._client_wrapper.httpx_client.request(
            "webhook_endpoints",
            method="POST",
            json={
                "url": url,
                "name": name,
                "status": status,
                "enabledEvents": enabled_events,
                "apiVersion": api_version,
                "advancedOptions": convert_and_respect_annotation_metadata(
                    object_=advanced_options, annotation=WebhookAdvancedOptionsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointCreate,
                    construct_type(
                        type_=WebhookEndpointCreate,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[WebhookEndpoint]:
        """
        Retrieve a webhook endpoint by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookEndpoint]
            Retrieved webhook endpoint
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpoint,
                    construct_type(
                        type_=WebhookEndpoint,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def update(
        self,
        id: str,
        *,
        url: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        enabled_events: typing.Optional[typing.Sequence[WebhookEndpointEventType]] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WebhookEndpoint]:
        """
        Update a webhook endpoint. Only the fields you include in the request body will be updated; omitted fields remain unchanged.

        The `apiVersion` of a webhook endpoint cannot be changed after creation.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to update.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        url : typing.Optional[str]
            The URL that webhook events will be sent to.

        name : typing.Optional[str]
            A human-readable name for the webhook endpoint.

        status : typing.Optional[WebhookEndpointStatus]

        enabled_events : typing.Optional[typing.Sequence[WebhookEndpointEventType]]
            The list of global event types to subscribe to.

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookEndpoint]
            Successfully updated webhook endpoint
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="POST",
            json={
                "url": url,
                "name": name,
                "status": status,
                "enabledEvents": enabled_events,
                "advancedOptions": convert_and_respect_annotation_metadata(
                    object_=advanced_options, annotation=WebhookAdvancedOptionsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpoint,
                    construct_type(
                        type_=WebhookEndpoint,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[WebhookEndpointsDeleteResponse]:
        """
        Delete a webhook endpoint and all of its subscriptions. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to delete.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WebhookEndpointsDeleteResponse]
            Successfully deleted webhook endpoint
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"webhook_endpoints/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WebhookEndpointsDeleteResponse,
                    construct_type(
                        type_=WebhookEndpointsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams(typing_extensions.TypedDict):
    x: float
    """
    X coordinate of the point
    """

    y: float
    """
    Y coordinate of the point
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .api_error import ApiErrorParams
    from .array_strategy import ArrayStrategyParams
    from .batch_processor_run import BatchProcessorRunParams
    from .batch_processor_run_metrics import BatchProcessorRunMetricsParams
    from .batch_processor_run_options import BatchProcessorRunOptionsParams
    from .batch_run import BatchRunParams
    from .batch_run_entity import (
        BatchRunEntityParams,
        BatchRunEntity_ClassifierParams,
        BatchRunEntity_ExtractorParams,
        BatchRunEntity_SplitterParams,
    )
    from .batch_run_entity_version import (
        BatchRunEntityVersionParams,
        BatchRunEntityVersion_ClassifierVersionParams,
        BatchRunEntityVersion_ExtractorVersionParams,
        BatchRunEntityVersion_SplitterVersionParams,
    )
    from .batch_run_metrics import BatchRunMetricsParams
    from .block import BlockParams
    from .block_details import BlockDetailsParams
    from .block_metadata import BlockMetadataParams
    from .block_metadata_page import BlockMetadataPageParams
    from .block_polygon_item import BlockPolygonItemParams
    from .bounding_box import BoundingBoxParams
    from .chunk import ChunkParams
    from .chunk_metadata import ChunkMetadataParams
    from .chunk_metadata_page_range import ChunkMetadataPageRangeParams
    from .citation import CitationParams
    from .citation_page import CitationPageParams
    from .classification import ClassificationParams
    from .classifications import ClassificationsParams
    from .classifier import ClassifierParams
    from .classifier_created_webhook_event import ClassifierCreatedWebhookEventParams
    from .classifier_deleted_webhook_event import ClassifierDeletedWebhookEventParams
    from .classifier_draft_updated_webhook_event import ClassifierDraftUpdatedWebhookEventParams
    from .classifier_summary import ClassifierSummaryParams
    from .classifier_updated_webhook_event import ClassifierUpdatedWebhookEventParams
    from .classifier_version import ClassifierVersionParams
    from .classifier_version_published_webhook_event import ClassifierVersionPublishedWebhookEventParams
    from .classifier_version_summary import ClassifierVersionSummaryParams
    from .classify_advanced_options import ClassifyAdvancedOptionsParams
    from .classify_config import ClassifyConfigParams
    from .classify_output import ClassifyOutputParams
    from .classify_override_config import ClassifyOverrideConfigParams
    from .classify_request_classifier import ClassifyRequestClassifierParams
    from .classify_request_file import ClassifyRequestFileParams
    from .classify_result import ClassifyResultParams
    from .classify_run import ClassifyRunParams
    from .classify_run_failed_webhook_event import ClassifyRunFailedWebhookEventParams
    from .classify_run_processed_webhook_event import ClassifyRunProcessedWebhookEventParams
    from .classify_run_summary import ClassifyRunSummaryParams
    from .classify_step_run import ClassifyStepRunParams
    from .classify_step_run_step import ClassifyStepRunStepParams
    from .conditional_extract_result import ConditionalExtractResultParams
    from .conditional_extract_step_run import ConditionalExtractStepRunParams
    from .conditional_extract_step_run_step import ConditionalExtractStepRunStepParams
    from .edit_bounding_box import EditBoundingBoxParams
    from .edit_config import EditConfigParams
    from .edit_config_advanced_options import EditConfigAdvancedOptionsParams
    from .edit_json import EditJsonParams
    from .edit_object_json import EditObjectJsonParams
    from .edit_request_file import EditRequestFileParams
    from .edit_root_json import EditRootJsonParams
    from .edit_run import EditRunParams
    from .edit_run_failed_webhook_event import EditRunFailedWebhookEventParams
    from .edit_run_metrics import EditRunMetricsParams
    from .edit_run_output import EditRunOutputParams
    from .edit_run_output_edited_file import EditRunOutputEditedFileParams
    from .edit_run_processed_webhook_event import EditRunProcessedWebhookEventParams
    from .edit_text_options import EditTextOptionsParams
    from .enum import EnumParams
    from .enum_option import EnumOptionParams
    from .evaluation_set import EvaluationSetParams
    from .evaluation_set_entity import (
        EvaluationSetEntityParams,
        EvaluationSetEntity_ClassifierParams,
        EvaluationSetEntity_ExtractorParams,
        EvaluationSetEntity_SplitterParams,
    )
    from .evaluation_set_item import EvaluationSetItemParams
    from .evaluation_set_item_summary import EvaluationSetItemSummaryParams
    from .evaluation_set_run import EvaluationSetRunParams
    from .evaluation_set_run_entity import (
        EvaluationSetRunEntityParams,
        EvaluationSetRunEntity_ClassifierParams,
        EvaluationSetRunEntity_ExtractorParams,
        EvaluationSetRunEntity_SplitterParams,
    )
    from .evaluation_set_run_entity_version import (
        EvaluationSetRunEntityVersionParams,
        EvaluationSetRunEntityVersion_ClassifierVersionParams,
        EvaluationSetRunEntityVersion_ExtractorVersionParams,
        EvaluationSetRunEntityVersion_SplitterVersionParams,
    )
    from .evaluation_set_run_metrics import EvaluationSetRunMetricsParams
    from .evaluation_set_run_options import EvaluationSetRunOptionsParams
    from .excel_sheet_range import ExcelSheetRangeParams
    from .external_data_validation_result import ExternalDataValidationResultParams
    from .external_data_validation_result_response import ExternalDataValidationResultResponseParams
    from .external_data_validation_step_run import ExternalDataValidationStepRunParams
    from .external_data_validation_step_run_step import ExternalDataValidationStepRunStepParams
    from .extract_advanced_options import ExtractAdvancedOptionsParams
    from .extract_advanced_options_review_agent import ExtractAdvancedOptionsReviewAgentParams
    from .extract_chunking_options import ExtractChunkingOptionsParams
    from .extract_config import ExtractConfigParams
    from .extract_config_json import ExtractConfigJsonParams
    from .extract_config_legacy import ExtractConfigLegacyParams
    from .extract_output import ExtractOutputParams
    from .extract_output_edits import ExtractOutputEditsParams
    from .extract_output_json import ExtractOutputJsonParams
    from .extract_output_legacy import ExtractOutputLegacyParams
    from .extract_output_metadata import ExtractOutputMetadataParams
    from .extract_output_metadata_value import ExtractOutputMetadataValueParams
    from .extract_override_config_json import ExtractOverrideConfigJsonParams
    from .extract_request_extractor import ExtractRequestExtractorParams
    from .extract_request_file import ExtractRequestFileParams
    from .extract_result import ExtractResultParams
    from .extract_run import ExtractRunParams
    from .extract_run_failed_webhook_event import ExtractRunFailedWebhookEventParams
    from .extract_run_processed_webhook_event import ExtractRunProcessedWebhookEventParams
    from .extract_run_summary import ExtractRunSummaryParams
    from .extract_step_run import ExtractStepRunParams
    from .extract_step_run_step import ExtractStepRunStepParams
    from .extraction_field import ExtractionFieldParams
    from .extraction_field_result import ExtractionFieldResultParams
    from .extraction_field_result_reference import ExtractionFieldResultReferenceParams
    from .extraction_field_result_reference_bounding_boxes_item import (
        ExtractionFieldResultReferenceBoundingBoxesItemParams,
    )
    from .extractor import ExtractorParams
    from .extractor_created_webhook_event import ExtractorCreatedWebhookEventParams
    from .extractor_deleted_webhook_event import ExtractorDeletedWebhookEventParams
    from .extractor_draft_updated_webhook_event import ExtractorDraftUpdatedWebhookEventParams
    from .extractor_summary import ExtractorSummaryParams
    from .extractor_updated_webhook_event import ExtractorUpdatedWebhookEventParams
    from .extractor_version import ExtractorVersionParams
    from .extractor_version_published_webhook_event import ExtractorVersionPublishedWebhookEventParams
    from .extractor_version_summary import ExtractorVersionSummaryParams
    from .figure_details import FigureDetailsParams
    from .file import FileParams
    from .file_contents import FileContentsParams
    from .file_contents_pages_item import FileContentsPagesItemParams
    from .file_contents_sections_item import FileContentsSectionsItemParams
    from .file_contents_sheets_item import FileContentsSheetsItemParams
    from .file_from_id import FileFromIdParams
    from .file_from_text import FileFromTextParams
    from .file_from_url import FileFromUrlParams
    from .file_metadata import FileMetadataParams
    from .file_summary import FileSummaryParams
    from .insight import InsightParams
    from .legacy_batch_processor_run import LegacyBatchProcessorRunParams
    from .legacy_batch_processor_run_metrics import LegacyBatchProcessorRunMetricsParams
    from .legacy_batch_processor_run_options import LegacyBatchProcessorRunOptionsParams
    from .legacy_classification import LegacyClassificationParams
    from .legacy_classification_advanced_options import LegacyClassificationAdvancedOptionsParams
    from .legacy_classification_config import LegacyClassificationConfigParams
    from .legacy_classifier_output import LegacyClassifierOutputParams
    from .legacy_document_processor_run_credits import LegacyDocumentProcessorRunCreditsParams
    from .legacy_error import LegacyErrorParams
    from .legacy_extend_error import LegacyExtendErrorParams
    from .legacy_extract_chunking_options import LegacyExtractChunkingOptionsParams
    from .legacy_extraction_advanced_options import LegacyExtractionAdvancedOptionsParams
    from .legacy_extraction_advanced_options_review_agent import LegacyExtractionAdvancedOptionsReviewAgentParams
    from .legacy_extraction_config import LegacyExtractionConfigParams
    from .legacy_extraction_field import LegacyExtractionFieldParams
    from .legacy_extraction_field_result import LegacyExtractionFieldResultParams
    from .legacy_extraction_output import LegacyExtractionOutputParams
    from .legacy_extraction_output_edits import LegacyExtractionOutputEditsParams
    from .legacy_fields_array_output import LegacyFieldsArrayOutputParams
    from .legacy_json_output import LegacyJsonOutputParams
    from .legacy_json_output_metadata_value import LegacyJsonOutputMetadataValueParams
    from .legacy_json_output_metadata_value_citations_item import LegacyJsonOutputMetadataValueCitationsItemParams
    from .legacy_json_output_metadata_value_citations_item_polygon_item import (
        LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams,
    )
    from .legacy_list_processors_processor import LegacyListProcessorsProcessorParams
    from .legacy_list_processors_processor_version import LegacyListProcessorsProcessorVersionParams
    from .legacy_list_processors_response import LegacyListProcessorsResponseParams
    from .legacy_processor import LegacyProcessorParams
    from .legacy_processor_output import LegacyProcessorOutputParams
    from .legacy_processor_run import LegacyProcessorRunParams
    from .legacy_processor_run_config import (
        LegacyProcessorRunConfigParams,
        LegacyProcessorRunConfig_ClassifyParams,
        LegacyProcessorRunConfig_ExtractParams,
        LegacyProcessorRunConfig_SplitterParams,
    )
    from .legacy_processor_run_file_input import LegacyProcessorRunFileInputParams
    from .legacy_processor_run_merged_processors_item import LegacyProcessorRunMergedProcessorsItemParams
    from .legacy_processor_run_summary import LegacyProcessorRunSummaryParams
    from .legacy_processor_version import LegacyProcessorVersionParams
    from .legacy_processor_version_config import (
        LegacyProcessorVersionConfigParams,
        LegacyProcessorVersionConfig_ClassifyParams,
        LegacyProcessorVersionConfig_ExtractParams,
        LegacyProcessorVersionConfig_SplitterParams,
    )
    from .legacy_provided_classifier_output import LegacyProvidedClassifierOutputParams
    from .legacy_provided_extraction_field_result import LegacyProvidedExtractionFieldResultParams
    from .legacy_provided_extraction_output import LegacyProvidedExtractionOutputParams
    from .legacy_provided_fields_array_output import LegacyProvidedFieldsArrayOutputParams
    from .legacy_provided_json_output import LegacyProvidedJsonOutputParams
    from .legacy_provided_processor_output import LegacyProvidedProcessorOutputParams
    from .legacy_provided_splitter_output import LegacyProvidedSplitterOutputParams
    from .legacy_provided_splitter_output_splits_item import LegacyProvidedSplitterOutputSplitsItemParams
    from .legacy_splitter_advanced_options import LegacySplitterAdvancedOptionsParams
    from .legacy_splitter_config import LegacySplitterConfigParams
    from .legacy_splitter_output import LegacySplitterOutputParams
    from .legacy_splitter_output_splits_item import LegacySplitterOutputSplitsItemParams
    from .merge_extract_result import MergeExtractResultParams
    from .merge_extract_result_merged_extractors_item import MergeExtractResultMergedExtractorsItemParams
    from .merge_extract_step_run import MergeExtractStepRunParams
    from .merge_extract_step_run_step import MergeExtractStepRunStepParams
    from .page_ranges import PageRangesParams
    from .page_ranges_item import PageRangesItemParams
    from .parent_split import ParentSplitParams
    from .parse_config import ParseConfigParams
    from .parse_config_advanced_options import ParseConfigAdvancedOptionsParams
    from .parse_config_advanced_options_return_ocr import ParseConfigAdvancedOptionsReturnOcrParams
    from .parse_config_block_options import ParseConfigBlockOptionsParams
    from .parse_config_block_options_figures import ParseConfigBlockOptionsFiguresParams
    from .parse_config_block_options_tables import ParseConfigBlockOptionsTablesParams
    from .parse_config_block_options_tables_agentic import ParseConfigBlockOptionsTablesAgenticParams
    from .parse_config_block_options_text import ParseConfigBlockOptionsTextParams
    from .parse_config_block_options_text_agentic import ParseConfigBlockOptionsTextAgenticParams
    from .parse_config_chunking_strategy import ParseConfigChunkingStrategyParams
    from .parse_config_chunking_strategy_options import ParseConfigChunkingStrategyOptionsParams
    from .parse_request_file import ParseRequestFileParams
    from .parse_result import ParseResultParams
    from .parse_run import ParseRunParams
    from .parse_run_failed_webhook_event import ParseRunFailedWebhookEventParams
    from .parse_run_metrics import ParseRunMetricsParams
    from .parse_run_output import ParseRunOutputParams
    from .parse_run_output_ocr import ParseRunOutputOcrParams
    from .parse_run_output_ocr_words_item import ParseRunOutputOcrWordsItemParams
    from .parse_run_processed_webhook_event import ParseRunProcessedWebhookEventParams
    from .parse_run_status import ParseRunStatusParams
    from .parse_step_run import ParseStepRunParams
    from .parse_step_run_step import ParseStepRunStepParams
    from .polygon import PolygonParams
    from .provided_classify_output import ProvidedClassifyOutputParams
    from .provided_extract_output import ProvidedExtractOutputParams
    from .provided_processor_output import ProvidedProcessorOutputParams
    from .provided_split_output import ProvidedSplitOutputParams
    from .provided_split_output_splits_item import ProvidedSplitOutputSplitsItemParams
    from .rule_validation_result import RuleValidationResultParams
    from .rule_validation_result_rules_item import RuleValidationResultRulesItemParams
    from .rule_validation_step_run import RuleValidationStepRunParams
    from .rule_validation_step_run_step import RuleValidationStepRunStepParams
    from .run_usage import RunUsageParams
    from .split_advanced_options import SplitAdvancedOptionsParams
    from .split_config import SplitConfigParams
    from .split_output import SplitOutputParams
    from .split_output_splits_item import SplitOutputSplitsItemParams
    from .split_override_config import SplitOverrideConfigParams
    from .split_request_file import SplitRequestFileParams
    from .split_request_splitter import SplitRequestSplitterParams
    from .split_result import SplitResultParams
    from .split_run import SplitRunParams
    from .split_run_failed_webhook_event import SplitRunFailedWebhookEventParams
    from .split_run_processed_webhook_event import SplitRunProcessedWebhookEventParams
    from .split_run_summary import SplitRunSummaryParams
    from .split_step_run import SplitStepRunParams
    from .split_step_run_step import SplitStepRunStepParams
    from .splitter import SplitterParams
    from .splitter_created_webhook_event import SplitterCreatedWebhookEventParams
    from .splitter_deleted_webhook_event import SplitterDeletedWebhookEventParams
    from .splitter_draft_updated_webhook_event import SplitterDraftUpdatedWebhookEventParams
    from .splitter_summary import SplitterSummaryParams
    from .splitter_updated_webhook_event import SplitterUpdatedWebhookEventParams
    from .splitter_version import SplitterVersionParams
    from .splitter_version_published_webhook_event import SplitterVersionPublishedWebhookEventParams
    from .splitter_version_summary import SplitterVersionSummaryParams
    from .step_run import StepRunParams
    from .step_run_base import StepRunBaseParams
    from .table_cell_details import TableCellDetailsParams
    from .table_details import TableDetailsParams
    from .too_many_requests_error_body import TooManyRequestsErrorBodyParams
    from .webhook_advanced_options import WebhookAdvancedOptionsParams
    from .webhook_endpoint import WebhookEndpointParams
    from .webhook_endpoint_create import WebhookEndpointCreateParams
    from .webhook_event import (
        WebhookEventParams,
        WebhookEvent_ClassifierCreatedParams,
        WebhookEvent_ClassifierDeletedParams,
        WebhookEvent_ClassifierDraftUpdatedParams,
        WebhookEvent_ClassifierUpdatedParams,
        WebhookEvent_ClassifierVersionPublishedParams,
        WebhookEvent_ClassifyRunFailedParams,
        WebhookEvent_ClassifyRunProcessedParams,
        WebhookEvent_EditRunFailedParams,
        WebhookEvent_EditRunProcessedParams,
        WebhookEvent_ExtractRunFailedParams,
        WebhookEvent_ExtractRunProcessedParams,
        WebhookEvent_ExtractorCreatedParams,
        WebhookEvent_ExtractorDeletedParams,
        WebhookEvent_ExtractorDraftUpdatedParams,
        WebhookEvent_ExtractorUpdatedParams,
        WebhookEvent_ExtractorVersionPublishedParams,
        WebhookEvent_ParseRunFailedParams,
        WebhookEvent_ParseRunProcessedParams,
        WebhookEvent_SplitRunFailedParams,
        WebhookEvent_SplitRunProcessedParams,
        WebhookEvent_SplitterCreatedParams,
        WebhookEvent_SplitterDeletedParams,
        WebhookEvent_SplitterDraftUpdatedParams,
        WebhookEvent_SplitterUpdatedParams,
        WebhookEvent_SplitterVersionPublishedParams,
        WebhookEvent_WorkflowCreatedParams,
        WebhookEvent_WorkflowDeletedParams,
        WebhookEvent_WorkflowDeployedParams,
        WebhookEvent_WorkflowRunCancelledParams,
        WebhookEvent_WorkflowRunCompletedParams,
        WebhookEvent_WorkflowRunFailedParams,
        WebhookEvent_WorkflowRunNeedsReviewParams,
        WebhookEvent_WorkflowRunRejectedParams,
        WebhookEvent_WorkflowRunStepRunProcessedParams,
    )
    from .webhook_payload_options import WebhookPayloadOptionsParams
    from .webhook_subscription import WebhookSubscriptionParams
    from .workflow import WorkflowParams
    from .workflow_created_webhook_event import WorkflowCreatedWebhookEventParams
    from .workflow_deleted_webhook_event import WorkflowDeletedWebhookEventParams
    from .workflow_deployed_webhook_event import WorkflowDeployedWebhookEventParams
    from .workflow_reference import WorkflowReferenceParams
    from .workflow_run import WorkflowRunParams
    from .workflow_run_cancelled_webhook_event import WorkflowRunCancelledWebhookEventParams
    from .workflow_run_completed_webhook_event import WorkflowRunCompletedWebhookEventParams
    from .workflow_run_failed_webhook_event import WorkflowRunFailedWebhookEventParams
    from .workflow_run_needs_review_webhook_event import WorkflowRunNeedsReviewWebhookEventParams
    from .workflow_run_rejected_webhook_event import WorkflowRunRejectedWebhookEventParams
    from .workflow_run_step_run_processed_webhook_event import WorkflowRunStepRunProcessedWebhookEventParams
    from .workflow_run_summary import WorkflowRunSummaryParams
    from .workflow_step_base import WorkflowStepBaseParams
    from .workflow_summary import WorkflowSummaryParams
    from .workflow_version import WorkflowVersionParams
    from .workflow_version_summary import WorkflowVersionSummaryParams
_dynamic_imports: typing.Dict[str, str] = {
    "ApiErrorParams": ".api_error",
    "ArrayStrategyParams": ".array_strategy",
    "BatchProcessorRunMetricsParams": ".batch_processor_run_metrics",
    "BatchProcessorRunOptionsParams": ".batch_processor_run_options",
    "BatchProcessorRunParams": ".batch_processor_run",
    "BatchRunEntityParams": ".batch_run_entity",
    "BatchRunEntityVersionParams": ".batch_run_entity_version",
    "BatchRunEntityVersion_ClassifierVersionParams": ".batch_run_entity_version",
    "BatchRunEntityVersion_ExtractorVersionParams": ".batch_run_entity_version",
    "BatchRunEntityVersion_SplitterVersionParams": ".batch_run_entity_version",
    "BatchRunEntity_ClassifierParams": ".batch_run_entity",
    "BatchRunEntity_ExtractorParams": ".batch_run_entity",
    "BatchRunEntity_SplitterParams": ".batch_run_entity",
    "BatchRunMetricsParams": ".batch_run_metrics",
    "BatchRunParams": ".batch_run",
    "BlockDetailsParams": ".block_details",
    "BlockMetadataPageParams": ".block_metadata_page",
    "BlockMetadataParams": ".block_metadata",
    "BlockParams": ".block",
    "BlockPolygonItemParams": ".block_polygon_item",
    "BoundingBoxParams": ".bounding_box",
    "ChunkMetadataPageRangeParams": ".chunk_metadata_page_range",
    "ChunkMetadataParams": ".chunk_metadata",
    "ChunkParams": ".chunk",
    "CitationPageParams": ".citation_page",
    "CitationParams": ".citation",
    "ClassificationParams": ".classification",
    "ClassificationsParams": ".classifications",
    "ClassifierCreatedWebhookEventParams": ".classifier_created_webhook_event",
    "ClassifierDeletedWebhookEventParams": ".classifier_deleted_webhook_event",
    "ClassifierDraftUpdatedWebhookEventParams": ".classifier_draft_updated_webhook_event",
    "ClassifierParams": ".classifier",
    "ClassifierSummaryParams": ".classifier_summary",
    "ClassifierUpdatedWebhookEventParams": ".classifier_updated_webhook_event",
    "ClassifierVersionParams": ".classifier_version",
    "ClassifierVersionPublishedWebhookEventParams": ".classifier_version_published_webhook_event",
    "ClassifierVersionSummaryParams": ".classifier_version_summary",
    "ClassifyAdvancedOptionsParams": ".classify_advanced_options",
    "ClassifyConfigParams": ".classify_config",
    "ClassifyOutputParams": ".classify_output",
    "ClassifyOverrideConfigParams": ".classify_override_config",
    "ClassifyRequestClassifierParams": ".classify_request_classifier",
    "ClassifyRequestFileParams": ".classify_request_file",
    "ClassifyResultParams": ".classify_result",
    "ClassifyRunFailedWebhookEventParams": ".classify_run_failed_webhook_event",
    "ClassifyRunParams": ".classify_run",
    "ClassifyRunProcessedWebhookEventParams": ".classify_run_processed_webhook_event",
    "ClassifyRunSummaryParams": ".classify_run_summary",
    "ClassifyStepRunParams": ".classify_step_run",
    "ClassifyStepRunStepParams": ".classify_step_run_step",
    "ConditionalExtractResultParams": ".conditional_extract_result",
    "ConditionalExtractStepRunParams": ".conditional_extract_step_run",
    "ConditionalExtractStepRunStepParams": ".conditional_extract_step_run_step",
    "EditBoundingBoxParams": ".edit_bounding_box",
    "EditConfigAdvancedOptionsParams": ".edit_config_advanced_options",
    "EditConfigParams": ".edit_config",
    "EditJsonParams": ".edit_json",
    "EditObjectJsonParams": ".edit_object_json",
    "EditRequestFileParams": ".edit_request_file",
    "EditRootJsonParams": ".edit_root_json",
    "EditRunFailedWebhookEventParams": ".edit_run_failed_webhook_event",
    "EditRunMetricsParams": ".edit_run_metrics",
    "EditRunOutputEditedFileParams": ".edit_run_output_edited_file",
    "EditRunOutputParams": ".edit_run_output",
    "EditRunParams": ".edit_run",
    "EditRunProcessedWebhookEventParams": ".edit_run_processed_webhook_event",
    "EditTextOptionsParams": ".edit_text_options",
    "EnumOptionParams": ".enum_option",
    "EnumParams": ".enum",
    "EvaluationSetEntityParams": ".evaluation_set_entity",
    "EvaluationSetEntity_ClassifierParams": ".evaluation_set_entity",
    "EvaluationSetEntity_ExtractorParams": ".evaluation_set_entity",
    "EvaluationSetEntity_SplitterParams": ".evaluation_set_entity",
    "EvaluationSetItemParams": ".evaluation_set_item",
    "EvaluationSetItemSummaryParams": ".evaluation_set_item_summary",
    "EvaluationSetParams": ".evaluation_set",
    "EvaluationSetRunEntityParams": ".evaluation_set_run_entity",
    "EvaluationSetRunEntityVersionParams": ".evaluation_set_run_entity_version",
    "EvaluationSetRunEntityVersion_ClassifierVersionParams": ".evaluation_set_run_entity_version",
    "EvaluationSetRunEntityVersion_ExtractorVersionParams": ".evaluation_set_run_entity_version",
    "EvaluationSetRunEntityVersion_SplitterVersionParams": ".evaluation_set_run_entity_version",
    "EvaluationSetRunEntity_ClassifierParams": ".evaluation_set_run_entity",
    "EvaluationSetRunEntity_ExtractorParams": ".evaluation_set_run_entity",
    "EvaluationSetRunEntity_SplitterParams": ".evaluation_set_run_entity",
    "EvaluationSetRunMetricsParams": ".evaluation_set_run_metrics",
    "EvaluationSetRunOptionsParams": ".evaluation_set_run_options",
    "EvaluationSetRunParams": ".evaluation_set_run",
    "ExcelSheetRangeParams": ".excel_sheet_range",
    "ExternalDataValidationResultParams": ".external_data_validation_result",
    "ExternalDataValidationResultResponseParams": ".external_data_validation_result_response",
    "ExternalDataValidationStepRunParams": ".external_data_validation_step_run",
    "ExternalDataValidationStepRunStepParams": ".external_data_validation_step_run_step",
    "ExtractAdvancedOptionsParams": ".extract_advanced_options",
    "ExtractAdvancedOptionsReviewAgentParams": ".extract_advanced_options_review_agent",
    "ExtractChunkingOptionsParams": ".extract_chunking_options",
    "ExtractConfigJsonParams": ".extract_config_json",
    "ExtractConfigLegacyParams": ".extract_config_legacy",
    "ExtractConfigParams": ".extract_config",
    "ExtractOutputEditsParams": ".extract_output_edits",
    "ExtractOutputJsonParams": ".extract_output_json",
    "ExtractOutputLegacyParams": ".extract_output_legacy",
    "ExtractOutputMetadataParams": ".extract_output_metadata",
    "ExtractOutputMetadataValueParams": ".extract_output_metadata_value",
    "ExtractOutputParams": ".extract_output",
    "ExtractOverrideConfigJsonParams": ".extract_override_config_json",
    "ExtractRequestExtractorParams": ".extract_request_extractor",
    "ExtractRequestFileParams": ".extract_request_file",
    "ExtractResultParams": ".extract_result",
    "ExtractRunFailedWebhookEventParams": ".extract_run_failed_webhook_event",
    "ExtractRunParams": ".extract_run",
    "ExtractRunProcessedWebhookEventParams": ".extract_run_processed_webhook_event",
    "ExtractRunSummaryParams": ".extract_run_summary",
    "ExtractStepRunParams": ".extract_step_run",
    "ExtractStepRunStepParams": ".extract_step_run_step",
    "ExtractionFieldParams": ".extraction_field",
    "ExtractionFieldResultParams": ".extraction_field_result",
    "ExtractionFieldResultReferenceBoundingBoxesItemParams": ".extraction_field_result_reference_bounding_boxes_item",
    "ExtractionFieldResultReferenceParams": ".extraction_field_result_reference",
    "ExtractorCreatedWebhookEventParams": ".extractor_created_webhook_event",
    "ExtractorDeletedWebhookEventParams": ".extractor_deleted_webhook_event",
    "ExtractorDraftUpdatedWebhookEventParams": ".extractor_draft_updated_webhook_event",
    "ExtractorParams": ".extractor",
    "ExtractorSummaryParams": ".extractor_summary",
    "ExtractorUpdatedWebhookEventParams": ".extractor_updated_webhook_event",
    "ExtractorVersionParams": ".extractor_version",
    "ExtractorVersionPublishedWebhookEventParams": ".extractor_version_published_webhook_event",
    "ExtractorVersionSummaryParams": ".extractor_version_summary",
    "FigureDetailsParams": ".figure_details",
    "FileContentsPagesItemParams": ".file_contents_pages_item",
    "FileContentsParams": ".file_contents",
    "FileContentsSectionsItemParams": ".file_contents_sections_item",
    "FileContentsSheetsItemParams": ".file_contents_sheets_item",
    "FileFromIdParams": ".file_from_id",
    "FileFromTextParams": ".file_from_text",
    "FileFromUrlParams": ".file_from_url",
    "FileMetadataParams": ".file_metadata",
    "FileParams": ".file",
    "FileSummaryParams": ".file_summary",
    "InsightParams": ".insight",
    "LegacyBatchProcessorRunMetricsParams": ".legacy_batch_processor_run_metrics",
    "LegacyBatchProcessorRunOptionsParams": ".legacy_batch_processor_run_options",
    "LegacyBatchProcessorRunParams": ".legacy_batch_processor_run",
    "LegacyClassificationAdvancedOptionsParams": ".legacy_classification_advanced_options",
    "LegacyClassificationConfigParams": ".legacy_classification_config",
    "LegacyClassificationParams": ".legacy_classification",
    "LegacyClassifierOutputParams": ".legacy_classifier_output",
    "LegacyDocumentProcessorRunCreditsParams": ".legacy_document_processor_run_credits",
    "LegacyErrorParams": ".legacy_error",
    "LegacyExtendErrorParams": ".legacy_extend_error",
    "LegacyExtractChunkingOptionsParams": ".legacy_extract_chunking_options",
    "LegacyExtractionAdvancedOptionsParams": ".legacy_extraction_advanced_options",
    "LegacyExtractionAdvancedOptionsReviewAgentParams": ".legacy_extraction_advanced_options_review_agent",
    "LegacyExtractionConfigParams": ".legacy_extraction_config",
    "LegacyExtractionFieldParams": ".legacy_extraction_field",
    "LegacyExtractionFieldResultParams": ".legacy_extraction_field_result",
    "LegacyExtractionOutputEditsParams": ".legacy_extraction_output_edits",
    "LegacyExtractionOutputParams": ".legacy_extraction_output",
    "LegacyFieldsArrayOutputParams": ".legacy_fields_array_output",
    "LegacyJsonOutputMetadataValueCitationsItemParams": ".legacy_json_output_metadata_value_citations_item",
    "LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams": ".legacy_json_output_metadata_value_citations_item_polygon_item",
    "LegacyJsonOutputMetadataValueParams": ".legacy_json_output_metadata_value",
    "LegacyJsonOutputParams": ".legacy_json_output",
    "LegacyListProcessorsProcessorParams": ".legacy_list_processors_processor",
    "LegacyListProcessorsProcessorVersionParams": ".legacy_list_processors_processor_version",
    "LegacyListProcessorsResponseParams": ".legacy_list_processors_response",
    "LegacyProcessorOutputParams": ".legacy_processor_output",
    "LegacyProcessorParams": ".legacy_processor",
    "LegacyProcessorRunConfigParams": ".legacy_processor_run_config",
    "LegacyProcessorRunConfig_ClassifyParams": ".legacy_processor_run_config",
    "LegacyProcessorRunConfig_ExtractParams": ".legacy_processor_run_config",
    "LegacyProcessorRunConfig_SplitterParams": ".legacy_processor_run_config",
    "LegacyProcessorRunFileInputParams": ".legacy_processor_run_file_input",
    "LegacyProcessorRunMergedProcessorsItemParams": ".legacy_processor_run_merged_processors_item",
    "LegacyProcessorRunParams": ".legacy_processor_run",
    "LegacyProcessorRunSummaryParams": ".legacy_processor_run_summary",
    "LegacyProcessorVersionConfigParams": ".legacy_processor_version_config",
    "LegacyProcessorVersionConfig_ClassifyParams": ".legacy_processor_version_config",
    "LegacyProcessorVersionConfig_ExtractParams": ".legacy_processor_version_config",
    "LegacyProcessorVersionConfig_SplitterParams": ".legacy_processor_version_config",
    "LegacyProcessorVersionParams": ".legacy_processor_version",
    "LegacyProvidedClassifierOutputParams": ".legacy_provided_classifier_output",
    "LegacyProvidedExtractionFieldResultParams": ".legacy_provided_extraction_field_result",
    "LegacyProvidedExtractionOutputParams": ".legacy_provided_extraction_output",
    "LegacyProvidedFieldsArrayOutputParams": ".legacy_provided_fields_array_output",
    "LegacyProvidedJsonOutputParams": ".legacy_provided_json_output",
    "LegacyProvidedProcessorOutputParams": ".legacy_provided_processor_output",
    "LegacyProvidedSplitterOutputParams": ".legacy_provided_splitter_output",
    "LegacyProvidedSplitterOutputSplitsItemParams": ".legacy_provided_splitter_output_splits_item",
    "LegacySplitterAdvancedOptionsParams": ".legacy_splitter_advanced_options",
    "LegacySplitterConfigParams": ".legacy_splitter_config",
    "LegacySplitterOutputParams": ".legacy_splitter_output",
    "LegacySplitterOutputSplitsItemParams": ".legacy_splitter_output_splits_item",
    "MergeExtractResultMergedExtractorsItemParams": ".merge_extract_result_merged_extractors_item",
    "MergeExtractResultParams": ".merge_extract_result",
    "MergeExtractStepRunParams": ".merge_extract_step_run",
    "MergeExtractStepRunStepParams": ".merge_extract_step_run_step",
    "PageRangesItemParams": ".page_ranges_item",
    "PageRangesParams": ".page_ranges",
    "ParentSplitParams": ".parent_split",
    "ParseConfigAdvancedOptionsParams": ".parse_config_advanced_options",
    "ParseConfigAdvancedOptionsReturnOcrParams": ".parse_config_advanced_options_return_ocr",
    "ParseConfigBlockOptionsFiguresParams": ".parse_config_block_options_figures",
    "ParseConfigBlockOptionsParams": ".parse_config_block_options",
    "ParseConfigBlockOptionsTablesAgenticParams": ".parse_config_block_options_tables_agentic",
    "ParseConfigBlockOptionsTablesParams": ".parse_config_block_options_tables",
    "ParseConfigBlockOptionsTextAgenticParams": ".parse_config_block_options_text_agentic",
    "ParseConfigBlockOptionsTextParams": ".parse_config_block_options_text",
    "ParseConfigChunkingStrategyOptionsParams": ".parse_config_chunking_strategy_options",
    "ParseConfigChunkingStrategyParams": ".parse_config_chunking_strategy",
    "ParseConfigParams": ".parse_config",
    "ParseRequestFileParams": ".parse_request_file",
    "ParseResultParams": ".parse_result",
    "ParseRunFailedWebhookEventParams": ".parse_run_failed_webhook_event",
    "ParseRunMetricsParams": ".parse_run_metrics",
    "ParseRunOutputOcrParams": ".parse_run_output_ocr",
    "ParseRunOutputOcrWordsItemParams": ".parse_run_output_ocr_words_item",
    "ParseRunOutputParams": ".parse_run_output",
    "ParseRunParams": ".parse_run",
    "ParseRunProcessedWebhookEventParams": ".parse_run_processed_webhook_event",
    "ParseRunStatusParams": ".parse_run_status",
    "ParseStepRunParams": ".parse_step_run",
    "ParseStepRunStepParams": ".parse_step_run_step",
    "PolygonParams": ".polygon",
    "ProvidedClassifyOutputParams": ".provided_classify_output",
    "ProvidedExtractOutputParams": ".provided_extract_output",
    "ProvidedProcessorOutputParams": ".provided_processor_output",
    "ProvidedSplitOutputParams": ".provided_split_output",
    "ProvidedSplitOutputSplitsItemParams": ".provided_split_output_splits_item",
    "RuleValidationResultParams": ".rule_validation_result",
    "RuleValidationResultRulesItemParams": ".rule_validation_result_rules_item",
    "RuleValidationStepRunParams": ".rule_validation_step_run",
    "RuleValidationStepRunStepParams": ".rule_validation_step_run_step",
    "RunUsageParams": ".run_usage",
    "SplitAdvancedOptionsParams": ".split_advanced_options",
    "SplitConfigParams": ".split_config",
    "SplitOutputParams": ".split_output",
    "SplitOutputSplitsItemParams": ".split_output_splits_item",
    "SplitOverrideConfigParams": ".split_override_config",
    "SplitRequestFileParams": ".split_request_file",
    "SplitRequestSplitterParams": ".split_request_splitter",
    "SplitResultParams": ".split_result",
    "SplitRunFailedWebhookEventParams": ".split_run_failed_webhook_event",
    "SplitRunParams": ".split_run",
    "SplitRunProcessedWebhookEventParams": ".split_run_processed_webhook_event",
    "SplitRunSummaryParams": ".split_run_summary",
    "SplitStepRunParams": ".split_step_run",
    "SplitStepRunStepParams": ".split_step_run_step",
    "SplitterCreatedWebhookEventParams": ".splitter_created_webhook_event",
    "SplitterDeletedWebhookEventParams": ".splitter_deleted_webhook_event",
    "SplitterDraftUpdatedWebhookEventParams": ".splitter_draft_updated_webhook_event",
    "SplitterParams": ".splitter",
    "SplitterSummaryParams": ".splitter_summary",
    "SplitterUpdatedWebhookEventParams": ".splitter_updated_webhook_event",
    "SplitterVersionParams": ".splitter_version",
    "SplitterVersionPublishedWebhookEventParams": ".splitter_version_published_webhook_event",
    "SplitterVersionSummaryParams": ".splitter_version_summary",
    "StepRunBaseParams": ".step_run_base",
    "StepRunParams": ".step_run",
    "TableCellDetailsParams": ".table_cell_details",
    "TableDetailsParams": ".table_details",
    "TooManyRequestsErrorBodyParams": ".too_many_requests_error_body",
    "WebhookAdvancedOptionsParams": ".webhook_advanced_options",
    "WebhookEndpointCreateParams": ".webhook_endpoint_create",
    "WebhookEndpointParams": ".webhook_endpoint",
    "WebhookEventParams": ".webhook_event",
    "WebhookEvent_ClassifierCreatedParams": ".webhook_event",
    "WebhookEvent_ClassifierDeletedParams": ".webhook_event",
    "WebhookEvent_ClassifierDraftUpdatedParams": ".webhook_event",
    "WebhookEvent_ClassifierUpdatedParams": ".webhook_event",
    "WebhookEvent_ClassifierVersionPublishedParams": ".webhook_event",
    "WebhookEvent_ClassifyRunFailedParams": ".webhook_event",
    "WebhookEvent_ClassifyRunProcessedParams": ".webhook_event",
    "WebhookEvent_EditRunFailedParams": ".webhook_event",
    "WebhookEvent_EditRunProcessedParams": ".webhook_event",
    "WebhookEvent_ExtractRunFailedParams": ".webhook_event",
    "WebhookEvent_ExtractRunProcessedParams": ".webhook_event",
    "WebhookEvent_ExtractorCreatedParams": ".webhook_event",
    "WebhookEvent_ExtractorDeletedParams": ".webhook_event",
    "WebhookEvent_ExtractorDraftUpdatedParams": ".webhook_event",
    "WebhookEvent_ExtractorUpdatedParams": ".webhook_event",
    "WebhookEvent_ExtractorVersionPublishedParams": ".webhook_event",
    "WebhookEvent_ParseRunFailedParams": ".webhook_event",
    "WebhookEvent_ParseRunProcessedParams": ".webhook_event",
    "WebhookEvent_SplitRunFailedParams": ".webhook_event",
    "WebhookEvent_SplitRunProcessedParams": ".webhook_event",
    "WebhookEvent_SplitterCreatedParams": ".webhook_event",
    "WebhookEvent_SplitterDeletedParams": ".webhook_event",
    "WebhookEvent_SplitterDraftUpdatedParams": ".webhook_event",
    "WebhookEvent_SplitterUpdatedParams": ".webhook_event",
    "WebhookEvent_SplitterVersionPublishedParams": ".webhook_event",
    "WebhookEvent_WorkflowCreatedParams": ".webhook_event",
    "WebhookEvent_WorkflowDeletedParams": ".webhook_event",
    "WebhookEvent_WorkflowDeployedParams": ".webhook_event",
    "WebhookEvent_WorkflowRunCancelledParams": ".webhook_event",
    "WebhookEvent_WorkflowRunCompletedParams": ".webhook_event",
    "WebhookEvent_WorkflowRunFailedParams": ".webhook_event",
    "WebhookEvent_WorkflowRunNeedsReviewParams": ".webhook_event",
    "WebhookEvent_WorkflowRunRejectedParams": ".webhook_event",
    "WebhookEvent_WorkflowRunStepRunProcessedParams": ".webhook_event",
    "WebhookPayloadOptionsParams": ".webhook_payload_options",
    "WebhookSubscriptionParams": ".webhook_subscription",
    "WorkflowCreatedWebhookEventParams": ".workflow_created_webhook_event",
    "WorkflowDeletedWebhookEventParams": ".workflow_deleted_webhook_event",
    "WorkflowDeployedWebhookEventParams": ".workflow_deployed_webhook_event",
    "WorkflowParams": ".workflow",
    "WorkflowReferenceParams": ".workflow_reference",
    "WorkflowRunCancelledWebhookEventParams": ".workflow_run_cancelled_webhook_event",
    "WorkflowRunCompletedWebhookEventParams": ".workflow_run_completed_webhook_event",
    "WorkflowRunFailedWebhookEventParams": ".workflow_run_failed_webhook_event",
    "WorkflowRunNeedsReviewWebhookEventParams": ".workflow_run_needs_review_webhook_event",
    "WorkflowRunParams": ".workflow_run",
    "WorkflowRunRejectedWebhookEventParams": ".workflow_run_rejected_webhook_event",
    "WorkflowRunStepRunProcessedWebhookEventParams": ".workflow_run_step_run_processed_webhook_event",
    "WorkflowRunSummaryParams": ".workflow_run_summary",
    "WorkflowStepBaseParams": ".workflow_step_base",
    "WorkflowSummaryParams": ".workflow_summary",
    "WorkflowVersionParams": ".workflow_version",
    "WorkflowVersionSummaryParams": ".workflow_version_summary",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ApiErrorParams",
    "ArrayStrategyParams",
    "BatchProcessorRunMetricsParams",
    "BatchProcessorRunOptionsParams",
    "BatchProcessorRunParams",
    "BatchRunEntityParams",
    "BatchRunEntityVersionParams",
    "BatchRunEntityVersion_ClassifierVersionParams",
    "BatchRunEntityVersion_ExtractorVersionParams",
    "BatchRunEntityVersion_SplitterVersionParams",
    "BatchRunEntity_ClassifierParams",
    "BatchRunEntity_ExtractorParams",
    "BatchRunEntity_SplitterParams",
    "BatchRunMetricsParams",
    "BatchRunParams",
    "BlockDetailsParams",
    "BlockMetadataPageParams",
    "BlockMetadataParams",
    "BlockParams",
    "BlockPolygonItemParams",
    "BoundingBoxParams",
    "ChunkMetadataPageRangeParams",
    "ChunkMetadataParams",
    "ChunkParams",
    "CitationPageParams",
    "CitationParams",
    "ClassificationParams",
    "ClassificationsParams",
    "ClassifierCreatedWebhookEventParams",
    "ClassifierDeletedWebhookEventParams",
    "ClassifierDraftUpdatedWebhookEventParams",
    "ClassifierParams",
    "ClassifierSummaryParams",
    "ClassifierUpdatedWebhookEventParams",
    "ClassifierVersionParams",
    "ClassifierVersionPublishedWebhookEventParams",
    "ClassifierVersionSummaryParams",
    "ClassifyAdvancedOptionsParams",
    "ClassifyConfigParams",
    "ClassifyOutputParams",
    "ClassifyOverrideConfigParams",
    "ClassifyRequestClassifierParams",
    "ClassifyRequestFileParams",
    "ClassifyResultParams",
    "ClassifyRunFailedWebhookEventParams",
    "ClassifyRunParams",
    "ClassifyRunProcessedWebhookEventParams",
    "ClassifyRunSummaryParams",
    "ClassifyStepRunParams",
    "ClassifyStepRunStepParams",
    "ConditionalExtractResultParams",
    "ConditionalExtractStepRunParams",
    "ConditionalExtractStepRunStepParams",
    "EditBoundingBoxParams",
    "EditConfigAdvancedOptionsParams",
    "EditConfigParams",
    "EditJsonParams",
    "EditObjectJsonParams",
    "EditRequestFileParams",
    "EditRootJsonParams",
    "EditRunFailedWebhookEventParams",
    "EditRunMetricsParams",
    "EditRunOutputEditedFileParams",
    "EditRunOutputParams",
    "EditRunParams",
    "EditRunProcessedWebhookEventParams",
    "EditTextOptionsParams",
    "EnumOptionParams",
    "EnumParams",
    "EvaluationSetEntityParams",
    "EvaluationSetEntity_ClassifierParams",
    "EvaluationSetEntity_ExtractorParams",
    "EvaluationSetEntity_SplitterParams",
    "EvaluationSetItemParams",
    "EvaluationSetItemSummaryParams",
    "EvaluationSetParams",
    "EvaluationSetRunEntityParams",
    "EvaluationSetRunEntityVersionParams",
    "EvaluationSetRunEntityVersion_ClassifierVersionParams",
    "EvaluationSetRunEntityVersion_ExtractorVersionParams",
    "EvaluationSetRunEntityVersion_SplitterVersionParams",
    "EvaluationSetRunEntity_ClassifierParams",
    "EvaluationSetRunEntity_ExtractorParams",
    "EvaluationSetRunEntity_SplitterParams",
    "EvaluationSetRunMetricsParams",
    "EvaluationSetRunOptionsParams",
    "EvaluationSetRunParams",
    "ExcelSheetRangeParams",
    "ExternalDataValidationResultParams",
    "ExternalDataValidationResultResponseParams",
    "ExternalDataValidationStepRunParams",
    "ExternalDataValidationStepRunStepParams",
    "ExtractAdvancedOptionsParams",
    "ExtractAdvancedOptionsReviewAgentParams",
    "ExtractChunkingOptionsParams",
    "ExtractConfigJsonParams",
    "ExtractConfigLegacyParams",
    "ExtractConfigParams",
    "ExtractOutputEditsParams",
    "ExtractOutputJsonParams",
    "ExtractOutputLegacyParams",
    "ExtractOutputMetadataParams",
    "ExtractOutputMetadataValueParams",
    "ExtractOutputParams",
    "ExtractOverrideConfigJsonParams",
    "ExtractRequestExtractorParams",
    "ExtractRequestFileParams",
    "ExtractResultParams",
    "ExtractRunFailedWebhookEventParams",
    "ExtractRunParams",
    "ExtractRunProcessedWebhookEventParams",
    "ExtractRunSummaryParams",
    "ExtractStepRunParams",
    "ExtractStepRunStepParams",
    "ExtractionFieldParams",
    "ExtractionFieldResultParams",
    "ExtractionFieldResultReferenceBoundingBoxesItemParams",
    "ExtractionFieldResultReferenceParams",
    "ExtractorCreatedWebhookEventParams",
    "ExtractorDeletedWebhookEventParams",
    "ExtractorDraftUpdatedWebhookEventParams",
    "ExtractorParams",
    "ExtractorSummaryParams",
    "ExtractorUpdatedWebhookEventParams",
    "ExtractorVersionParams",
    "ExtractorVersionPublishedWebhookEventParams",
    "ExtractorVersionSummaryParams",
    "FigureDetailsParams",
    "FileContentsPagesItemParams",
    "FileContentsParams",
    "FileContentsSectionsItemParams",
    "FileContentsSheetsItemParams",
    "FileFromIdParams",
    "FileFromTextParams",
    "FileFromUrlParams",
    "FileMetadataParams",
    "FileParams",
    "FileSummaryParams",
    "InsightParams",
    "LegacyBatchProcessorRunMetricsParams",
    "LegacyBatchProcessorRunOptionsParams",
    "LegacyBatchProcessorRunParams",
    "LegacyClassificationAdvancedOptionsParams",
    "LegacyClassificationConfigParams",
    "LegacyClassificationParams",
    "LegacyClassifierOutputParams",
    "LegacyDocumentProcessorRunCreditsParams",
    "LegacyErrorParams",
    "LegacyExtendErrorParams",
    "LegacyExtractChunkingOptionsParams",
    "LegacyExtractionAdvancedOptionsParams",
    "LegacyExtractionAdvancedOptionsReviewAgentParams",
    "LegacyExtractionConfigParams",
    "LegacyExtractionFieldParams",
    "LegacyExtractionFieldResultParams",
    "LegacyExtractionOutputEditsParams",
    "LegacyExtractionOutputParams",
    "LegacyFieldsArrayOutputParams",
    "LegacyJsonOutputMetadataValueCitationsItemParams",
    "LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams",
    "LegacyJsonOutputMetadataValueParams",
    "LegacyJsonOutputParams",
    "LegacyListProcessorsProcessorParams",
    "LegacyListProcessorsProcessorVersionParams",
    "LegacyListProcessorsResponseParams",
    "LegacyProcessorOutputParams",
    "LegacyProcessorParams",
    "LegacyProcessorRunConfigParams",
    "LegacyProcessorRunConfig_ClassifyParams",
    "LegacyProcessorRunConfig_ExtractParams",
    "LegacyProcessorRunConfig_SplitterParams",
    "LegacyProcessorRunFileInputParams",
    "LegacyProcessorRunMergedProcessorsItemParams",
    "LegacyProcessorRunParams",
    "LegacyProcessorRunSummaryParams",
    "LegacyProcessorVersionConfigParams",
    "LegacyProcessorVersionConfig_ClassifyParams",
    "LegacyProcessorVersionConfig_ExtractParams",
    "LegacyProcessorVersionConfig_SplitterParams",
    "LegacyProcessorVersionParams",
    "LegacyProvidedClassifierOutputParams",
    "LegacyProvidedExtractionFieldResultParams",
    "LegacyProvidedExtractionOutputParams",
    "LegacyProvidedFieldsArrayOutputParams",
    "LegacyProvidedJsonOutputParams",
    "LegacyProvidedProcessorOutputParams",
    "LegacyProvidedSplitterOutputParams",
    "LegacyProvidedSplitterOutputSplitsItemParams",
    "LegacySplitterAdvancedOptionsParams",
    "LegacySplitterConfigParams",
    "LegacySplitterOutputParams",
    "LegacySplitterOutputSplitsItemParams",
    "MergeExtractResultMergedExtractorsItemParams",
    "MergeExtractResultParams",
    "MergeExtractStepRunParams",
    "MergeExtractStepRunStepParams",
    "PageRangesItemParams",
    "PageRangesParams",
    "ParentSplitParams",
    "ParseConfigAdvancedOptionsParams",
    "ParseConfigAdvancedOptionsReturnOcrParams",
    "ParseConfigBlockOptionsFiguresParams",
    "ParseConfigBlockOptionsParams",
    "ParseConfigBlockOptionsTablesAgenticParams",
    "ParseConfigBlockOptionsTablesParams",
    "ParseConfigBlockOptionsTextAgenticParams",
    "ParseConfigBlockOptionsTextParams",
    "ParseConfigChunkingStrategyOptionsParams",
    "ParseConfigChunkingStrategyParams",
    "ParseConfigParams",
    "ParseRequestFileParams",
    "ParseResultParams",
    "ParseRunFailedWebhookEventParams",
    "ParseRunMetricsParams",
    "ParseRunOutputOcrParams",
    "ParseRunOutputOcrWordsItemParams",
    "ParseRunOutputParams",
    "ParseRunParams",
    "ParseRunProcessedWebhookEventParams",
    "ParseRunStatusParams",
    "ParseStepRunParams",
    "ParseStepRunStepParams",
    "PolygonParams",
    "ProvidedClassifyOutputParams",
    "ProvidedExtractOutputParams",
    "ProvidedProcessorOutputParams",
    "ProvidedSplitOutputParams",
    "ProvidedSplitOutputSplitsItemParams",
    "RuleValidationResultParams",
    "RuleValidationResultRulesItemParams",
    "RuleValidationStepRunParams",
    "RuleValidationStepRunStepParams",
    "RunUsageParams",
    "SplitAdvancedOptionsParams",
    "SplitConfigParams",
    "SplitOutputParams",
    "SplitOutputSplitsItemParams",
    "SplitOverrideConfigParams",
    "SplitRequestFileParams",
    "SplitRequestSplitterParams",
    "SplitResultParams",
    "SplitRunFailedWebhookEventParams",
    "SplitRunParams",
    "SplitRunProcessedWebhookEventParams",
    "SplitRunSummaryParams",
    "SplitStepRunParams",
    "SplitStepRunStepParams",
    "SplitterCreatedWebhookEventParams",
    "SplitterDeletedWebhookEventParams",
    "SplitterDraftUpdatedWebhookEventParams",
    "SplitterParams",
    "SplitterSummaryParams",
    "SplitterUpdatedWebhookEventParams",
    "SplitterVersionParams",
    "SplitterVersionPublishedWebhookEventParams",
    "SplitterVersionSummaryParams",
    "StepRunBaseParams",
    "StepRunParams",
    "TableCellDetailsParams",
    "TableDetailsParams",
    "TooManyRequestsErrorBodyParams",
    "WebhookAdvancedOptionsParams",
    "WebhookEndpointCreateParams",
    "WebhookEndpointParams",
    "WebhookEventParams",
    "WebhookEvent_ClassifierCreatedParams",
    "WebhookEvent_ClassifierDeletedParams",
    "WebhookEvent_ClassifierDraftUpdatedParams",
    "WebhookEvent_ClassifierUpdatedParams",
    "WebhookEvent_ClassifierVersionPublishedParams",
    "WebhookEvent_ClassifyRunFailedParams",
    "WebhookEvent_ClassifyRunProcessedParams",
    "WebhookEvent_EditRunFailedParams",
    "WebhookEvent_EditRunProcessedParams",
    "WebhookEvent_ExtractRunFailedParams",
    "WebhookEvent_ExtractRunProcessedParams",
    "WebhookEvent_ExtractorCreatedParams",
    "WebhookEvent_ExtractorDeletedParams",
    "WebhookEvent_ExtractorDraftUpdatedParams",
    "WebhookEvent_ExtractorUpdatedParams",
    "WebhookEvent_ExtractorVersionPublishedParams",
    "WebhookEvent_ParseRunFailedParams",
    "WebhookEvent_ParseRunProcessedParams",
    "WebhookEvent_SplitRunFailedParams",
    "WebhookEvent_SplitRunProcessedParams",
    "WebhookEvent_SplitterCreatedParams",
    "WebhookEvent_SplitterDeletedParams",
    "WebhookEvent_SplitterDraftUpdatedParams",
    "WebhookEvent_SplitterUpdatedParams",
    "WebhookEvent_SplitterVersionPublishedParams",
    "WebhookEvent_WorkflowCreatedParams",
    "WebhookEvent_WorkflowDeletedParams",
    "WebhookEvent_WorkflowDeployedParams",
    "WebhookEvent_WorkflowRunCancelledParams",
    "WebhookEvent_WorkflowRunCompletedParams",
    "WebhookEvent_WorkflowRunFailedParams",
    "WebhookEvent_WorkflowRunNeedsReviewParams",
    "WebhookEvent_WorkflowRunRejectedParams",
    "WebhookEvent_WorkflowRunStepRunProcessedParams",
    "WebhookPayloadOptionsParams",
    "WebhookSubscriptionParams",
    "WorkflowCreatedWebhookEventParams",
    "WorkflowDeletedWebhookEventParams",
    "WorkflowDeployedWebhookEventParams",
    "WorkflowParams",
    "WorkflowReferenceParams",
    "WorkflowRunCancelledWebhookEventParams",
    "WorkflowRunCompletedWebhookEventParams",
    "WorkflowRunFailedWebhookEventParams",
    "WorkflowRunNeedsReviewWebhookEventParams",
    "WorkflowRunParams",
    "WorkflowRunRejectedWebhookEventParams",
    "WorkflowRunStepRunProcessedWebhookEventParams",
    "WorkflowRunSummaryParams",
    "WorkflowStepBaseParams",
    "WorkflowSummaryParams",
    "WorkflowVersionParams",
    "WorkflowVersionSummaryParams",
]



"""
Webhook utilities for signature verification and event parsing.

Example:
    from extend_ai import Extend

    client = Extend(token="...")

    @app.post("/webhook")
    def handle_webhook(request):
        try:
            event = client.webhooks.verify_and_parse(
                body=request.body.decode(),
                headers=dict(request.headers),
                signing_secret="wss_your_signing_secret"
            )

            if event.event_type == "workflow_run.completed":
                print("Workflow completed:", event.payload)

            return {"status": "ok"}
        except WebhookSignatureVerificationError:
            return {"error": "Invalid signature"}, 401
"""

import hashlib
import hmac
import json
import logging
import time
from dataclasses import dataclass
from typing import TYPE_CHECKING, Any, Dict, Optional, Union

import httpx

from .errors import SignedUrlNotAllowedError, WebhookParseError, WebhookPayloadFetchError, WebhookSignatureVerificationError

if TYPE_CHECKING:
    from ..types.webhook_event import WebhookEvent

logger = logging.getLogger(__name__)

# Default timeout for HTTP requests when fetching signed URL payloads (in seconds)
_DEFAULT_FETCH_TIMEOUT = 30.0


@dataclass
class SignedDataUrlPayload:
    """Payload structure when webhook is delivered via signed URL (for large payloads)."""

    data: str  # The signed URL to fetch the full payload (expires in 1 hour)
    id: str  # The ID of the run/resource
    object: str  # Discriminator indicating this is a signed URL payload ("signed_data_url")
    metadata: Optional[Dict[str, Any]] = None  # Optional metadata passed when the run was created


@dataclass
class WebhookEventWithSignedUrl:
    """Webhook event when payload is delivered via signed URL (before fetching)."""

    event_id: str  # Unique identifier for this webhook event
    event_type: str  # The type of event (e.g., "workflow_run.completed")
    payload: SignedDataUrlPayload  # The signed URL payload - use fetch_signed_payload() to get the full data


# Union type representing either a normal webhook event or one with a signed URL payload.
# When parsing succeeds, you get a typed WebhookEvent; when it's a signed URL, you get
# WebhookEventWithSignedUrl; for unknown event types, you get a raw dict fallback.
RawWebhookEvent = Union["WebhookEvent", WebhookEventWithSignedUrl, Dict[str, Any]]


def _is_signed_data_url_payload(payload: Any) -> bool:
    """Check if a payload is a signed URL payload."""
    if not isinstance(payload, dict):
        return False
    return payload.get("object") == "signed_data_url" and isinstance(payload.get("data"), str)


def _build_signed_url_event(event_data: Dict[str, Any], payload: Dict[str, Any]) -> WebhookEventWithSignedUrl:
    """Build a WebhookEventWithSignedUrl from raw event data and its signed URL payload."""
    return WebhookEventWithSignedUrl(
        event_id=event_data.get("eventId", ""),
        event_type=event_data.get("eventType", ""),
        payload=SignedDataUrlPayload(
            data=payload.get("data", ""),
            id=payload.get("id", ""),
            object=payload.get("object", ""),
            metadata=payload.get("metadata"),
        ),
    )


def _normalize_body(body: Union[str, bytes]) -> str:
    """Normalize body to a string, decoding bytes if necessary."""
    if isinstance(body, bytes):
        return body.decode("utf-8")
    return body


class Webhooks:
    """
    Webhook utilities for signature verification and event parsing.

    Example:
        client = Extend(token="...")

        # Simple usage - verify and parse in one step
        event = client.webhooks.verify_and_parse(body, headers, secret)

        # Or verify first, then parse separately
        if client.webhooks.verify(body, headers, secret):
            event = client.webhooks.parse(body)
            # handle event
    """

    def verify_and_parse(
        self,
        body: Union[str, bytes],
        headers: Dict[str, Any],
        signing_secret: str,
        max_age_seconds: int = 300,
        allow_signed_url: bool = False,
    ) -> RawWebhookEvent:
        """
        Verifies the webhook signature and parses the event.

        By default, this method returns a WebhookEvent. If the webhook contains a
        signed URL payload (used for large payloads), it throws a SignedUrlNotAllowedError.

        To handle signed URL payloads, pass allow_signed_url=True. Use is_signed_url_event()
        to check if you received a signed URL, then call fetch_signed_payload() to get the
        full payload.

        Args:
            body: The raw request body as a string or bytes
            headers: The request headers (must include x-extend-request-timestamp and
                    x-extend-request-signature)
            signing_secret: Your webhook signing secret (starts with wss_)
            max_age_seconds: Maximum age of the request in seconds (default: 300 = 5 minutes).
                            Set to 0 to disable timestamp validation.
            allow_signed_url: Whether to allow signed URL payloads. Default: False.
                            When False: Raises SignedUrlNotAllowedError if a signed URL payload
                            is received.
                            When True: Returns RawWebhookEvent (may be WebhookEventWithSignedUrl).

        Returns:
            The verified and parsed webhook event

        Raises:
            WebhookSignatureVerificationError: If signature verification fails
            WebhookParseError: If the body cannot be parsed as JSON
            SignedUrlNotAllowedError: If a signed URL payload is received without
                                     allow_signed_url=True

        Example:
            # Simple usage (most users) - raises if signed URL received
            event = client.webhooks.verify_and_parse(body, headers, secret)

            # Handle signed URL payloads
            event = client.webhooks.verify_and_parse(body, headers, secret, allow_signed_url=True)
            if client.webhooks.is_signed_url_event(event):
                # Check metadata before fetching (e.g., environment check)
                if event.payload.metadata.get("env") == "production":
                    # Use the async version in async contexts:
                    #   full_event = await client.webhooks.fetch_signed_payload(event)
                    # Or use the sync version:
                    full_event = client.webhooks.fetch_signed_payload_sync(event)
                    # handle full_event
            else:
                # Normal inline payload
                # handle event
        """
        body_str = _normalize_body(body)

        # Verify the signature
        self._verify_signature(body_str, headers, signing_secret, max_age_seconds)

        # Parse the event
        try:
            event_data = json.loads(body_str)
        except json.JSONDecodeError as e:
            raise WebhookParseError(f"Failed to parse webhook body as JSON: {e}")

        # Check if it's a signed URL payload
        payload = event_data.get("payload", {})
        if _is_signed_data_url_payload(payload):
            if not allow_signed_url:
                raise SignedUrlNotAllowedError()
            return _build_signed_url_event(event_data, payload)

        # Return typed event when possible, fall back to raw dict for unknown event types
        return self._try_parse_webhook_event(event_data)

    def verify(
        self,
        body: Union[str, bytes],
        headers: Dict[str, Any],
        signing_secret: str,
        max_age_seconds: int = 300,
    ) -> bool:
        """
        Verifies a webhook signature without parsing the body.

        Args:
            body: The raw request body as a string or bytes
            headers: The request headers
            signing_secret: Your webhook signing secret
            max_age_seconds: Maximum age of the request in seconds (default: 300)

        Returns:
            True if the signature is valid, False otherwise

        Example:
            is_valid = client.webhooks.verify(body, headers, secret)
            if is_valid:
                event = client.webhooks.parse(body)
                # handle event
        """
        try:
            body_str = _normalize_body(body)
            self._verify_signature(body_str, headers, signing_secret, max_age_seconds)
            return True
        except WebhookSignatureVerificationError:
            return False

    def parse(self, body: Union[str, bytes]) -> RawWebhookEvent:
        """
        Parses a webhook event from a raw body without verification.
        Use this only if you've already verified the signature using verify().

        Args:
            body: The raw request body as a string or bytes

        Returns:
            The parsed webhook event (may be a signed URL event)

        Raises:
            WebhookParseError: If the body cannot be parsed as JSON

        Example:
            if client.webhooks.verify(body, headers, secret):
                event = client.webhooks.parse(body)
                if client.webhooks.is_signed_url_event(event):
                    full_event = client.webhooks.fetch_signed_payload_sync(event)
        """
        body_str = _normalize_body(body)

        try:
            event_data = json.loads(body_str)
        except json.JSONDecodeError as e:
            raise WebhookParseError(f"Failed to parse webhook body as JSON: {e}")

        # Check if it's a signed URL payload
        payload = event_data.get("payload", {})
        if _is_signed_data_url_payload(payload):
            return _build_signed_url_event(event_data, payload)

        return self._try_parse_webhook_event(event_data)

    async def fetch_signed_payload(self, event: WebhookEventWithSignedUrl) -> Any:
        """
        Fetches the full payload from a signed URL webhook event.

        Use this when you've received a WebhookEventWithSignedUrl (from verify_and_parse
        with allow_signed_url=True) and want to retrieve the full payload.

        Args:
            event: The webhook event with a signed URL payload

        Returns:
            The full webhook event with the resolved payload

        Raises:
            WebhookPayloadFetchError: If fetching the signed URL fails

        Example:
            event = client.webhooks.verify_and_parse(body, headers, secret, allow_signed_url=True)
            if client.webhooks.is_signed_url_event(event):
                # Check metadata before fetching (e.g., environment check)
                if event.payload.metadata.get("env") == "production":
                    full_event = await client.webhooks.fetch_signed_payload(event)
                    # full_event["payload"] is now the full WorkflowRun, ExtractRun, etc.
        """
        url = event.payload.data

        try:
            async with httpx.AsyncClient(timeout=_DEFAULT_FETCH_TIMEOUT) as client:
                response = await client.get(url)

                if not response.is_success:
                    raise WebhookPayloadFetchError(
                        f"Failed to fetch signed payload: {response.status_code} {response.reason_phrase}"
                    )

                full_payload = response.json()

                full_event = {
                    "eventId": event.event_id,
                    "eventType": event.event_type,
                    "payload": full_payload,
                }
                return self._try_parse_webhook_event(full_event)
        except WebhookPayloadFetchError:
            raise
        except Exception as e:
            raise WebhookPayloadFetchError(f"Failed to fetch signed payload: {e}")

    def fetch_signed_payload_sync(self, event: WebhookEventWithSignedUrl) -> Any:
        """
        Fetches the full payload from a signed URL webhook event (synchronous version).

        Args:
            event: The webhook event with a signed URL payload

        Returns:
            The full webhook event with the resolved payload

        Raises:
            WebhookPayloadFetchError: If fetching the signed URL fails
        """
        url = event.payload.data

        try:
            with httpx.Client(timeout=_DEFAULT_FETCH_TIMEOUT) as client:
                response = client.get(url)

                if not response.is_success:
                    raise WebhookPayloadFetchError(
                        f"Failed to fetch signed payload: {response.status_code} {response.reason_phrase}"
                    )

                full_payload = response.json()

                full_event = {
                    "eventId": event.event_id,
                    "eventType": event.event_type,
                    "payload": full_payload,
                }
                return self._try_parse_webhook_event(full_event)
        except WebhookPayloadFetchError:
            raise
        except Exception as e:
            raise WebhookPayloadFetchError(f"Failed to fetch signed payload: {e}")

    def is_signed_url_event(self, event: RawWebhookEvent) -> bool:
        """
        Check if a webhook event has a signed URL payload.

        Use this to determine if you need to call fetch_signed_payload() to get the
        full payload.

        Args:
            event: The webhook event to check

        Returns:
            True if the event has a signed URL payload

        Example:
            event = client.webhooks.verify_and_parse(body, headers, secret, allow_signed_url=True)
            if client.webhooks.is_signed_url_event(event):
                # event is WebhookEventWithSignedUrl
                print("Signed URL:", event.payload.data)
                print("Metadata:", event.payload.metadata)
            else:
                # event is WebhookEvent
                print("Full payload:", event.get("payload"))
        """
        if isinstance(event, WebhookEventWithSignedUrl):
            return True
        if isinstance(event, dict):
            return _is_signed_data_url_payload(event.get("payload", {}))
        return False

    def _try_parse_webhook_event(self, event_data: Dict[str, Any]) -> Any:
        """Try to parse as typed WebhookEvent, fall back to raw dict for unknown event types."""
        try:
            from ..types.webhook_event import WebhookEvent
            from ..core.pydantic_utilities import IS_PYDANTIC_V2

            if IS_PYDANTIC_V2:
                import pydantic

                adapter: pydantic.TypeAdapter[Any] = pydantic.TypeAdapter(WebhookEvent)
                return adapter.validate_python(event_data)
            else:
                import pydantic

                return pydantic.parse_obj_as(WebhookEvent, event_data)  # type: ignore[arg-type]
        except (ImportError, pydantic.ValidationError):  # type: ignore[union-attr]
            return event_data
        except Exception:
            logger.debug("Unexpected error parsing webhook event, falling back to raw dict", exc_info=True)
            return event_data

    def _verify_signature(
        self,
        body: str,
        headers: Dict[str, Any],
        signing_secret: str,
        max_age_seconds: int = 300,
    ) -> None:
        """Verifies the webhook signature. Raises on failure."""
        # Extract headers (handle case-insensitive lookup)
        timestamp = self._get_header(headers, "x-extend-request-timestamp")
        signature = self._get_header(headers, "x-extend-request-signature")

        if not timestamp:
            raise WebhookSignatureVerificationError("Missing x-extend-request-timestamp header")

        if not signature:
            raise WebhookSignatureVerificationError("Missing x-extend-request-signature header")

        if not signing_secret:
            raise WebhookSignatureVerificationError("Missing signing secret")

        # Validate timestamp to prevent replay attacks
        if max_age_seconds > 0:
            current_time = int(time.time())
            try:
                request_time = int(timestamp)
            except ValueError:
                raise WebhookSignatureVerificationError("Invalid timestamp format")

            age = current_time - request_time
            if age > max_age_seconds:
                raise WebhookSignatureVerificationError(
                    f"Request timestamp too old ({age}s > {max_age_seconds}s)"
                )

            if age < -60:  # Allow 1 minute clock skew
                raise WebhookSignatureVerificationError("Request timestamp in the future")

        # Compute expected signature
        message = f"v0:{timestamp}:{body}"
        expected_signature = hmac.new(
            signing_secret.encode("utf-8"),
            message.encode("utf-8"),
            hashlib.sha256,
        ).hexdigest()

        # Use timing-safe comparison to prevent timing attacks
        if not hmac.compare_digest(signature, expected_signature):
            raise WebhookSignatureVerificationError("Invalid signature")

    def _get_header(self, headers: Dict[str, Any], name: str) -> Optional[str]:
        """Get a header value (case-insensitive)."""
        lower_name = name.lower()
        for key, value in headers.items():
            if key.lower() == lower_name:
                if isinstance(value, list):
                    return value[0] if value else None
                return value
        return None



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class FileContentsSectionsItemParams(typing_extensions.TypedDict):
    start_page_number: typing_extensions.Annotated[int, FieldMetadata(alias="startPageNumber")]
    """
    The starting page number of this section.
    """

    end_page_number: typing_extensions.Annotated[int, FieldMetadata(alias="endPageNumber")]
    """
    The ending page number of this section.
    """

    markdown: str
    """
    The markdown content of this section.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .legacy_provided_classifier_output import LegacyProvidedClassifierOutput
from .legacy_provided_extraction_output import LegacyProvidedExtractionOutput
from .legacy_provided_splitter_output import LegacyProvidedSplitterOutput

LegacyProvidedProcessorOutput = typing.Union[
    LegacyProvidedExtractionOutput, LegacyProvidedClassifierOutput, LegacyProvidedSplitterOutput
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class ParseConfigChunkingStrategyOptionsParams(typing_extensions.TypedDict):
    """
    Additional options for the chunking strategy.
    """

    min_characters: typing_extensions.NotRequired[
        typing_extensions.Annotated[int, FieldMetadata(alias="minCharacters")]
    ]
    """
    Specify a minimum number of characters per chunk.
    """

    max_characters: typing_extensions.NotRequired[
        typing_extensions.Annotated[int, FieldMetadata(alias="maxCharacters")]
    ]
    """
    Specify a maximum number of characters per chunk.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ExtractConfigLegacyBaseProcessor(enum.StrEnum):
    """
    The base processor to use. For extractors, this can be either `"extraction_performance"` or `"extraction_light"`. Defaults to `"extraction_performance"` if not provided. See [Extraction Changelog](https://docs.extend.ai/2026-02-09/changelog/extraction/extraction-performance) for more details.
    """

    EXTRACTION_PERFORMANCE = "extraction_performance"
    EXTRACTION_LIGHT = "extraction_light"
    _UNKNOWN = "__EXTRACTCONFIGLEGACYBASEPROCESSOR_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ExtractConfigLegacyBaseProcessor":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        extraction_performance: typing.Callable[[], T_Result],
        extraction_light: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ExtractConfigLegacyBaseProcessor.EXTRACTION_PERFORMANCE:
            return extraction_performance()
        if self is ExtractConfigLegacyBaseProcessor.EXTRACTION_LIGHT:
            return extraction_light()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .classifications import Classifications
from .classify_advanced_options import ClassifyAdvancedOptions
from .classify_base_processor import ClassifyBaseProcessor
from .parse_config import ParseConfig


class ClassifyConfig(UncheckedBaseModel):
    base_processor: typing_extensions.Annotated[
        typing.Optional[ClassifyBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = pydantic.Field(alias="baseProcessor", default=None)
    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = (
        pydantic.Field(alias="baseVersion", default=None)
    )
    """
    The version of the `"classification_performance"` or `"classification_light"` processor to use. If not provided, the latest stable version for the selected `baseProcessor` will be used automatically. See [Classification Changelog](https://docs.extend.ai/2026-02-09/changelog/classification/classification-performance) for more details.
    """

    classifications: Classifications
    classification_rules: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="classificationRules")
    ] = pydantic.Field(alias="classificationRules", default=None)
    """
    Custom rules to guide the classification process in natural language.
    """

    advanced_options: typing_extensions.Annotated[
        typing.Optional[ClassifyAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = pydantic.Field(alias="advancedOptions", default=None)
    """
    Advanced configuration options.
    """

    parse_config: typing_extensions.Annotated[typing.Optional[ParseConfig], FieldMetadata(alias="parseConfig")] = (
        pydantic.Field(alias="parseConfig", default=None)
    )
    """
    Configuration options for the parsing process.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class FileFromTextParams(typing_extensions.TypedDict):
    """
    Raw text data to be processed. This will be used to create a `.txt` file.
    """

    text: str
    """
    A string containing raw text data to be processed.
    """

    name: typing_extensions.NotRequired[str]
    """
    The name of the created text file. If not set, the file name is generated.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .classify_result import ClassifyResultParams
from .classify_step_run_step import ClassifyStepRunStepParams
from .step_run_base import StepRunBaseParams


class ClassifyStepRunParams(StepRunBaseParams):
    step: ClassifyStepRunStepParams
    result: typing.Optional[ClassifyResultParams]
    """
    The result of this classify step. Currently, `result` is always present for returned step runs. It is nullable to allow for future support of returning step runs with non-successful statuses.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from .edit_config_advanced_options import EditConfigAdvancedOptionsParams
from .edit_root_json import EditRootJsonParams


class EditConfigParams(typing_extensions.TypedDict):
    """
    Configuration options for the editing process.
    """

    schema: typing_extensions.NotRequired[EditRootJsonParams]
    instructions: typing_extensions.NotRequired[str]
    """
    Custom instructions provided for the edit operation.
    """

    advanced_options: typing_extensions.NotRequired[
        typing_extensions.Annotated[EditConfigAdvancedOptionsParams, FieldMetadata(alias="advancedOptions")]
    ]
    """
    Advanced options for the edit operation.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .classifier_version import ClassifierVersion
from .created_at import CreatedAt
from .updated_at import UpdatedAt


class Classifier(UncheckedBaseModel):
    object: typing.Literal["classifier"] = pydantic.Field(default="classifier")
    """
    The type of object. Will always be `"classifier"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the classifier.
    
    Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    name: str = pydantic.Field()
    """
    The name of the classifier.
    
    Example: `"Document Type Classifier"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")] = pydantic.Field(
        alias="updatedAt"
    )
    draft_version: typing_extensions.Annotated[ClassifierVersion, FieldMetadata(alias="draftVersion")] = pydantic.Field(
        alias="draftVersion"
    )
    """
    The draft version of the classifier. This is the editable version in the Extend dashboard.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .created_at import CreatedAt
from .run_usage import RunUsage
from .updated_at import UpdatedAt
from .workflow_run_status import WorkflowRunStatus
from .workflow_summary import WorkflowSummary
from .workflow_version_summary import WorkflowVersionSummary


class WorkflowRunSummary(UncheckedBaseModel):
    object: typing.Literal["workflow_run"] = pydantic.Field(default="workflow_run")
    """
    The type of object. In this case, it will always be `"workflow_run"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the workflow run.
    
    Example: `"workflow_run_Zk9mNP12Qw4-yTv8BdR3H"`
    """

    status: WorkflowRunStatus
    workflow: WorkflowSummary
    workflow_version: typing_extensions.Annotated[WorkflowVersionSummary, FieldMetadata(alias="workflowVersion")] = (
        pydantic.Field(alias="workflowVersion")
    )
    dashboard_url: typing_extensions.Annotated[str, FieldMetadata(alias="dashboardUrl")] = pydantic.Field(
        alias="dashboardUrl"
    )
    """
    A URL to view this workflow run in the Extend dashboard.
    
    Example: `"https://dashboard.extend.ai/workflows/workflow_run_xKm9pNv3qWsY_jL2tR5Dh"`
    """

    reviewed_by_user: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="reviewedByUser")] = (
        pydantic.Field(alias="reviewedByUser", default=None)
    )
    """
    The user of the person who reviewed the workflow run. Will be null if the workflow run has not been reviewed.
    
    Example: `"jane.doe@example.com"`
    """

    reviewed_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="reviewedAt")] = (
        pydantic.Field(alias="reviewedAt", default=None)
    )
    """
    The time (in UTC) at which the workflow run was reviewed. Will follow the RFC 3339 format. Will be null if the workflow run has not been reviewed.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    initial_run_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="initialRunAt")] = (
        pydantic.Field(alias="initialRunAt", default=None)
    )
    """
    The time (in UTC) at which the workflow was initially created. Will follow the RFC 3339 format. Will be null if the run hasn't started yet.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    start_time: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="startTime")] = (
        pydantic.Field(alias="startTime", default=None)
    )
    """
    The start time (in UTC) that the workflow actually started executing. This occurs after the `initialRunAt` time. Will follow the RFC 3339 format. Will be null if not started.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    end_time: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="endTime")] = (
        pydantic.Field(alias="endTime", default=None)
    )
    """
    The end time (in UTC) that the workflow finished. Will follow the RFC 3339 format. Will be null if not finished.
    
    Example: `"2024-03-21T15:35:00Z"`
    """

    batch_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="batchId")] = pydantic.Field(
        alias="batchId", default=None
    )
    """
    The batch ID of the workflow run. If that workflow run was created from a batch of files, all runs in that batch will have the same batch ID.
    
    Example: `"batch_7Ws31-F5"`
    """

    rejection_note: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="rejectionNote")] = (
        pydantic.Field(alias="rejectionNote", default=None)
    )
    """
    The note that was added when the workflow run was rejected.
    
    Example: `"Invalid invoice format"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")] = pydantic.Field(
        alias="updatedAt"
    )
    usage: typing.Optional[RunUsage] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .classify_config import ClassifyConfig
from .created_at import CreatedAt


class ClassifierVersion(UncheckedBaseModel):
    object: typing.Literal["classifier_version"] = pydantic.Field(default="classifier_version")
    """
    The type of object. Will always be `"classifier_version"`.
    """

    id: str = pydantic.Field()
    """
    The unique identifier for this version of the classifier.
    
    Example: `"clv_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of this version of the classifier.
    
    Example: `"Added new document types for Q4 processing"`
    """

    version: str = pydantic.Field()
    """
    The version number or identifier for this specific version of the classifier. The draft version will have version="draft".
    
    Examples: `"1.0"`, `"2.1"`, `"draft"`
    """

    config: ClassifyConfig = pydantic.Field()
    """
    The configuration settings for this version of the classifier.
    """

    classifier_id: typing_extensions.Annotated[str, FieldMetadata(alias="classifierId")] = pydantic.Field(
        alias="classifierId"
    )
    """
    The ID of the classifier that this version belongs to.
    
    Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .extract_output_json import ExtractOutputJsonParams
from .extract_output_legacy import ExtractOutputLegacyParams

ExtractOutputParams = typing.Union[ExtractOutputJsonParams, ExtractOutputLegacyParams]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class LegacyExtractionFieldType(enum.StrEnum):
    """
    The type of the field.
    """

    STRING = "string"
    NUMBER = "number"
    CURRENCY = "currency"
    BOOLEAN = "boolean"
    DATE = "date"
    ARRAY = "array"
    ENUM = "enum"
    OBJECT = "object"
    SIGNATURE = "signature"
    _UNKNOWN = "__LEGACYEXTRACTIONFIELDTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "LegacyExtractionFieldType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        string: typing.Callable[[], T_Result],
        number: typing.Callable[[], T_Result],
        currency: typing.Callable[[], T_Result],
        boolean: typing.Callable[[], T_Result],
        date: typing.Callable[[], T_Result],
        array: typing.Callable[[], T_Result],
        enum: typing.Callable[[], T_Result],
        object: typing.Callable[[], T_Result],
        signature: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is LegacyExtractionFieldType.STRING:
            return string()
        if self is LegacyExtractionFieldType.NUMBER:
            return number()
        if self is LegacyExtractionFieldType.CURRENCY:
            return currency()
        if self is LegacyExtractionFieldType.BOOLEAN:
            return boolean()
        if self is LegacyExtractionFieldType.DATE:
            return date()
        if self is LegacyExtractionFieldType.ARRAY:
            return array()
        if self is LegacyExtractionFieldType.ENUM:
            return enum()
        if self is LegacyExtractionFieldType.OBJECT:
            return object()
        if self is LegacyExtractionFieldType.SIGNATURE:
            return signature()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...core.serialization import FieldMetadata


class WorkflowRunsCreateBatchResponseParams(typing_extensions.TypedDict):
    batch_id: typing_extensions.Annotated[str, FieldMetadata(alias="batchId")]
    """
    A unique identifier for the submitted batch. This ID can be used to correlate the workflow runs created by this request. You can find this `batchId` associated with individual runs when listing workflow runs or in webhook payloads.
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .webhook_subscription_event_type import WebhookSubscriptionEventType
from .webhook_subscription_resource_type import WebhookSubscriptionResourceType


class WebhookSubscription(UncheckedBaseModel):
    object: typing.Literal["webhook_subscription"] = "webhook_subscription"
    id: str = pydantic.Field()
    """
    Unique identifier for the webhook subscription.
    """

    webhook_endpoint_id: typing_extensions.Annotated[str, FieldMetadata(alias="webhookEndpointId")] = pydantic.Field(
        alias="webhookEndpointId"
    )
    """
    The ID of the webhook endpoint this subscription belongs to.
    """

    resource_type: typing_extensions.Annotated[WebhookSubscriptionResourceType, FieldMetadata(alias="resourceType")] = (
        pydantic.Field(alias="resourceType")
    )
    resource_id: typing_extensions.Annotated[str, FieldMetadata(alias="resourceId")] = pydantic.Field(
        alias="resourceId"
    )
    """
    The ID of the resource this subscription is scoped to (e.g., an extractor, classifier, splitter, or workflow).
    """

    enabled_events: typing_extensions.Annotated[
        typing.List[WebhookSubscriptionEventType], FieldMetadata(alias="enabledEvents")
    ] = pydantic.Field(alias="enabledEvents")
    """
    The list of event types this subscription is subscribed to. The valid event types depend on the `resourceType`.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    """
    The date and time the webhook subscription was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class ParentSplitParams(typing_extensions.TypedDict):
    """
    The split metadata details for a derivative file.

    **Availability:** Only included if this file is a derivative of another file, for instance if it was created via a Splitter in a workflow.
    """

    id: str
    """
    The ID of the split.
    """

    type: str
    """
    The type of the split.
    """

    identifier: str
    """
    The identifier of the split.
    """

    start_page: typing_extensions.Annotated[int, FieldMetadata(alias="startPage")]
    """
    The start page of the split.
    """

    end_page: typing_extensions.Annotated[int, FieldMetadata(alias="endPage")]
    """
    The end page of the split.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..types.legacy_extraction_field_result import LegacyExtractionFieldResult

LegacyFieldsArrayOutputParams = typing.Dict[str, LegacyExtractionFieldResult]
